using System;
using System.Collections.Generic;
using com.agilefaqs.rft.domain;
using com.agilefaqs.rft.suggestions.strategies;
using NUnit.Framework;
using Rhino.Mocks;

namespace com.agilefaqs.rft.suggestions
{
    [TestFixture]
    public class IdentityGeneratorTest
    {
        private const String FIRST_NAME = "Naresh";
        private const String LAST_NAME = "Jain";
        private const String AT = "@";

        private readonly Country country = new Country("IN", "INDIA", "Indian");

        private SuggestionStrategyFactory suggestionStrategyFactory;
        private IdentityGenerator IdentityGenerator;
        private ChildSuggestionFactory childSuggestionFactory;

        private IdentitySuggestionOneStrategy identitySuggestionOneStrategy;

        private IdentitySuggestionTwoStrategy identitySuggestionTwoStrategy;

        private IdentitySuggestionThreeStrategy identitySuggestionThreeStrategy;

        private IdentitySuggestionFourStrategy identitySuggestionFourStrategy;

        private readonly String suggestion =
            String.Concat(String.Concat(String.Concat(FIRST_NAME, AT), LAST_NAME), ".com");


        private SuggestionParam suggestionParams;

        private MockRepository m;

        [SetUp]
        public void setUp()
        {
            m = new MockRepository();
            childSuggestionFactory = m.Stub<ChildSuggestionFactory>();
            suggestionStrategyFactory = m.Stub<SuggestionStrategyFactory>(childSuggestionFactory);
            identitySuggestionOneStrategy = m.StrictMock<IdentitySuggestionOneStrategy>(childSuggestionFactory);
            identitySuggestionTwoStrategy = m.StrictMock<IdentitySuggestionTwoStrategy>(childSuggestionFactory);
            identitySuggestionThreeStrategy = m.StrictMock<IdentitySuggestionThreeStrategy>(childSuggestionFactory);
            identitySuggestionFourStrategy = m.StrictMock<IdentitySuggestionFourStrategy>(childSuggestionFactory);

            List<SuggestionStrategy> strategies = new List<SuggestionStrategy>();
            strategies.Add(identitySuggestionOneStrategy);
            strategies.Add(identitySuggestionTwoStrategy);
            strategies.Add(identitySuggestionThreeStrategy);
            strategies.Add(identitySuggestionFourStrategy);

            Expect.Call(suggestionStrategyFactory.getStrategies()).Return(strategies);
            m.Replay(suggestionStrategyFactory);
            IdentityGenerator = new IdentityGenerator(suggestionStrategyFactory);
            m.Verify(suggestionStrategyFactory);

            suggestionParams = new SuggestionParam(new LocationInformation(country, "mumbai"), FIRST_NAME, LAST_NAME);
        }

        [TearDown]
        public virtual void verifyMocks()
        {
            m.VerifyAll();
        }

        private void stopRecordingAndReplay()
        {
            m.ReplayAll();
        }

        [Test]
        public virtual void shouldGenerateFourSuggestionsIfAllStrategiesReturnsAIdentity()
        {
            Expect.Call(identitySuggestionOneStrategy.getIdentity(suggestionParams)).Return(suggestion);
            Expect.Call(identitySuggestionTwoStrategy.getIdentity(suggestionParams)).Return(suggestion);
            Expect.Call(identitySuggestionThreeStrategy.getIdentity(suggestionParams)).Return(suggestion);
            Expect.Call(identitySuggestionFourStrategy.getIdentity(suggestionParams)).Return(suggestion);
            stopRecordingAndReplay();
            List<string> ids = IdentityGenerator.getGeneratedIDs(suggestionParams);

            Assert.AreEqual(4, ids.Count);
        }

        [Test]
        public virtual void shouldGenerateFourSuggestionsIfAllStrategiesReturnsNull()
        {
            Expect.Call(identitySuggestionOneStrategy.getIdentity(suggestionParams)).Return(null);
            Expect.Call(identitySuggestionTwoStrategy.getIdentity(suggestionParams)).Return(null);
            Expect.Call(identitySuggestionThreeStrategy.getIdentity(suggestionParams)).Return(null);
            Expect.Call(identitySuggestionFourStrategy.getIdentity(suggestionParams)).Return(null);
            stopRecordingAndReplay();
            List<string> ids = IdentityGenerator.getGeneratedIDs(suggestionParams);

            Assert.AreEqual(0, ids.Count);
        }

        [Test]
        public virtual void shouldGenerateTwoSuggestionsIfTwoStrategiesReturnsNull()
        {
            Expect.Call(identitySuggestionOneStrategy.getIdentity(suggestionParams)).Return(suggestion);
            Expect.Call(identitySuggestionTwoStrategy.getIdentity(suggestionParams)).Return(suggestion);
            Expect.Call(identitySuggestionThreeStrategy.getIdentity(suggestionParams)).Return(null);
            Expect.Call(identitySuggestionFourStrategy.getIdentity(suggestionParams)).Return(null);
            stopRecordingAndReplay();
            List<string> ids = IdentityGenerator.getGeneratedIDs(suggestionParams);

            Assert.AreEqual(2, ids.Count);
        }
    }
}