using System;
using com.agilefaqs.rft.domain;
using NUnit.Framework;

namespace com.agilefaqs.rft.suggestions
{
    [TestFixture]
    public class SuggestionParamTest
    {
        private const String CITY = "mumbai";
        private const String COUNTRY_NAME = "india";
        private const String COUNTRY_MONIKER = "indian";
        private const String FIRST_NAME = "Naresh";
        private const String LAST_NAME = "Jain";
        private const String CITY2 = "blor";
        private readonly Country country = new Country("IN", COUNTRY_NAME, COUNTRY_MONIKER);

        [Test]
        public virtual void testEquals()
        {
            SuggestionParam paramOne =
                new SuggestionParam(new LocationInformation(country, CITY), FIRST_NAME, LAST_NAME);
            SuggestionParam paramTwo =
                new SuggestionParam(new LocationInformation(country, CITY2), FIRST_NAME, LAST_NAME);
            SuggestionParam paramThree =
                new SuggestionParam(new LocationInformation(country, CITY), FIRST_NAME, LAST_NAME);
            Assert.IsTrue(paramOne.Equals(paramThree));
            Assert.IsFalse(paramOne.Equals(paramTwo));
            Assert.IsFalse(paramThree.Equals(paramTwo));
            Assert.IsFalse(paramThree.Equals(null));
            Assert.IsFalse(paramThree.Equals(new User()));
        }

        [Test]
        public virtual void shouldTrimSpecialCharacters()
        {
            SuggestionParam paramOne =
                new SuggestionParam(new LocationInformation(country, CITY), "peppy*pa%", "#pat-el-");
            Assert.AreEqual(paramOne.ValidFirstNameToken, "peppypa");
            Assert.AreEqual(paramOne.ValidLastNameToken, "pat-el");
        }

        [Test]
        public virtual void shouldAppendLettersIfRequired()
        {
            SuggestionParam paramOne = new SuggestionParam(new LocationInformation(country, CITY), "pe%", "pa");
            Assert.AreEqual(paramOne.ValidFirstNameToken, "pepe");
            Assert.AreEqual(paramOne.ValidLastNameToken, "papa");
        }

        [Test]
        public virtual void truncateLettersIfRequired()
        {
            SuggestionParam paramOne =
                new SuggestionParam(new LocationInformation(country, CITY), "suchalongnameuhavegiven", "pa");
            Assert.AreEqual(paramOne.ValidFirstNameToken, "suchalongnameuh");
            Assert.AreEqual(paramOne.ValidLastNameToken, "papa");
        }
    }
}