using System;
using com.agilefaqs.rft.domain;

namespace com.agilefaqs.rft.suggestions.strategies
{
    public class RandomNumberBasedStrategy : ChildSuggestionStrategy
    {
        private UserService userService;

        private SuggestionsUtil utils;

        private RandomNumberGenerator randomNumberGenerator;

        public RandomNumberBasedStrategy(UserService userService, SuggestionsUtil utils,
                                         RandomNumberGenerator randomNumberGenerator)
        {
            this.userService = userService;
            this.utils = utils;
            this.randomNumberGenerator = randomNumberGenerator;
        }


        public virtual String generateId(ChildStrategyParam param)
        {
            String token1 = param.Token1;

            String token2 = param.Token2;
            IDTokens IdTokens = utils.getIdentityTokens(token1, token2);
            if (IdTokens == null)
            {
                return null;
            }
            String Identity = getIdentityBasedOnRandomNumber(IdTokens.Token1, IdTokens.Token2);
            return Identity;
        }

        protected internal virtual String getIdentityBasedOnRandomNumber(String token1, String token2)
        {
            String Identity = null;
            int attempt = 0;
            do
            {
                Identity = utils.appendTokensForId(String.Concat(token1, randomNumberGenerator.next()), token2);
            } while (!userService.isIdentityAvailable(Identity) && ++attempt != 5);
            if (attempt == 5)
                return null;
            return Identity;
        }
    }
}