using System;
using com.agilefaqs.rft.domain;
using com.agilefaqs.rft.utils;

namespace com.agilefaqs.rft.suggestions
{
    public class SuggestionParam
    {
        private readonly String userFirstName;
        private readonly String userLastName;
        private readonly LocationInformation locationInfo;
        private readonly String validFirstNameToken;
        private readonly String validLastNameToken;

        public virtual String City
        {
            get
            {
                if (locationInfo != null)
                {
                    return locationInfo.City;
                }
                return null;
            }
        }

        public virtual String CountryMoniker
        {
            get
            {
                if (locationInfo != null)
                {
                    return locationInfo.CountryMoniker;
                }
                return null;
            }
        }

        public virtual String CountryName
        {
            get
            {
                if (locationInfo != null)
                {
                    return locationInfo.CountryName;
                }
                return null;
            }
        }

        public virtual String ValidFirstNameToken
        {
            get { return validFirstNameToken; }
        }

        public virtual String ValidLastNameToken
        {
            get { return validLastNameToken; }
        }

        public virtual String UserFirstName
        {
            get { return userFirstName; }
        }

        public virtual String UserLastName
        {
            get { return userLastName; }
        }

        public virtual Country Country
        {
            get { return locationInfo.Country; }
        }

        public SuggestionParam(LocationInformation locationInfo, String userFirstName, String userLastName)
        {
            this.locationInfo = locationInfo;
            this.userFirstName = userFirstName;
            validFirstNameToken = getToken(userFirstName);
            this.userLastName = userLastName;
            validLastNameToken = getToken(userLastName);
        }

        private static String getToken(String token)
        {
            char[] allowedChars = new char[] {'-'};
            String word = StringUtils.removeSpecialCharactersIfAny(token, allowedChars);
            word = appendLettersIfRequired(word);
            word = truncateLettersIfRequired(word);
            return word;
        }

        private static String truncateLettersIfRequired(String word)
        {
            int maxLength = 15;
            if (word.Length > maxLength)
            {
                word = word.Substring(0, (maxLength) - (0));
            }
            return word;
        }

        private static String appendLettersIfRequired(String word)
        {
            while (word.Length < 3)
                word = String.Concat(word, word);
            return word;
        }


        public override bool Equals(Object other)
        {
            if (this == other)
            {
                return true;
            }
            if (other == null)
            {
                return false;
            }

            if (!(other is SuggestionParam))
            {
                return false;
            }
            return equals((SuggestionParam) other);
        }

        private bool equals(SuggestionParam other)
        {
            return
                userFirstName.Equals(other.userFirstName) && userLastName.Equals(other.userLastName) &&
                CountryName.Equals(other.CountryName) && City.Equals(other.City);
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }
}