using System;
using com.agilefaqs.rft.domain;

namespace com.agilefaqs.rft.suggestions
{
    public class SuggestionsUtil
    {
        private static readonly int MAX_ATTEMPTS = 5;

        private readonly DomainNameService domainNameService;

        public SuggestionsUtil(DomainNameService domainNameService)
        {
            this.domainNameService = domainNameService;
        }

        public virtual IDTokens getIdentityTokens(String token1, String token2)
        {
            if (isCelebrityName(token1, token2))
            {
                token1 = token1.Substring(0, (token1.Length - 1) - (0));
                token2 = token2.Substring(0, (token2.Length - 1) - (0));
            }
            int loopCounter = 1;
            do
            {
                loopCounter++;
                String generatedFirstToken = generateFirstToken(token1);
                String generatedSecondToken = generateSecondToken(token2);
                if (generatedFirstToken == null || generatedSecondToken == null)
                    return null;
                else if (isCelebrityName(generatedFirstToken, generatedSecondToken))
                {
                    token1 = generatedFirstToken.Substring(0, (generatedFirstToken.Length - 1) - (0));
                    token2 = generatedSecondToken.Substring(0, (generatedSecondToken.Length - 1) - (0));
                }
                else
                    return new IDTokens(generatedFirstToken, generatedSecondToken);
            } while (loopCounter != MAX_ATTEMPTS);

            return null;
        }

        private String generateSecondToken(String token2)
        {
            int loopCounter = 0;
            String restrictedWord;
            do
            {
                restrictedWord = domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(token2);
                if (restrictedWord != null)
                {
                    String replacement = restrictedWord.Substring(0, (restrictedWord.Length - 1) - (0));
                    token2 = token2.Replace(restrictedWord, replacement);
                    loopCounter++;
                }
            } while (restrictedWord != null && loopCounter != MAX_ATTEMPTS);

            if (loopCounter == MAX_ATTEMPTS)
                return null;
            return token2;
        }

        private String generateFirstToken(String token1)
        {
            int loopCounter = 0;
            String restrictedWord;
            do
            {
                restrictedWord = domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(token1);
                if (restrictedWord != null)
                {
                    String replacement = restrictedWord.Substring(0, (restrictedWord.Length - 1) - (0));
                    token1 = token1.Replace(restrictedWord, replacement);
                    loopCounter++;
                }
            } while (restrictedWord != null && loopCounter != MAX_ATTEMPTS);

            if (loopCounter == MAX_ATTEMPTS)
                return null;
            return token1;
        }

        private bool isCelebrityName(String token1, String token2)
        {
            return domainNameService.isCelebrityName(token1, token2);
        }

        public virtual String appendTokensForId(String token1, String token2)
        {
            return String.Concat(token1.ToLower() + "@", token2.ToLower()) + ".com";
        }
    }
}